// Drawing an Icosahedron with 20 triangular faces
#include <GL/glut.h>
#include <math.h>
#include <stdio.h>

#define x .525731112119133606
#define y .0
#define z .850650808352039932

int i,j ;    
GLfloat d1[3], d2[3], norm[3];

void normalize(float[]);
void normcrossprod(float[], float[], float[]);

void display (void)
{
    static GLfloat vdata[12][3] = {
        {-x,y,z}, {x,y,z}, {-x,y,-z}, {x,y,-z},
        {y,z,x}, {y,z,-x}, {y,-z,x}, {y,-z,-x},
        {z,x,y}, {-z,x,y}, {z,-x,y}, {-z,-x,y} };
 
    static GLuint tindices[20][3] = {
        {0,4,1}, {0,9,4}, {9,5,4}, {4,5,8}, {4,8,1}, 
        {8,10,1}, {8,3,10}, {5,3,8}, {5,2,3}, {2,7,3}, 
        {7,10,3}, {7,6,10}, {7,11,6}, {11,0,6}, {0,1,6}, 
        {6,1,10}, {9,0,11}, {9,11,2}, {9,2,5}, {7,2,11} };
    
    glClearColor (1.0, 1.0, 1.0, 0.0);    
    glClear(GL_COLOR_BUFFER_BIT);
    glColor3f(1.0, 1.0, 1.0);
    
//    glLineWidth(5.0);
    glPolygonMode(GL_FRONT, GL_FILL);
    glPolygonMode(GL_BACK, GL_LINE);

    glBegin(GL_TRIANGLES);
    for (i = 0; i < 20; i++) {

        for (j = 0; j < 3; j++) {
            d1[j] = vdata[tindices[i][0]][j] - vdata[tindices[i][1]][j];
            d2[j] = vdata[tindices[i][1]][j] - vdata[tindices[i][2]][j];
        }
        normcrossprod(d1, d2, norm);
        glNormal3fv(norm);

        glVertex3fv(&vdata[tindices[i][0]][0]);
        glVertex3fv(&vdata[tindices[i][1]][0]);
        glVertex3fv(&vdata[tindices[i][2]][0]);
    }
    glEnd();
    glFlush();
}

void normcrossprod(float v1[3], float v2[3], float out[3])
{
    GLfloat d;
    out[0] = v1[1]*v2[2] - v1[2]*v2[1];
    out[1] = v1[2]*v2[0] - v1[0]*v2[2];
    out[2] = v1[0]*v2[1] - v1[1]*v2[0];
    d = sqrt(out[0]*out[0]+out[1]*out[1]+out[2]*out[2]);
    out[0] /=d; out[1] /=d; out[2] /=d;
}

void init(void)
{
    GLfloat mat_specular[] = {1.0,1.0,1.0,0.0};
    GLfloat mat_shininess[] = {20.0};
    GLfloat light0_diffuse[] = {1.0,1.0,1.0,1.0};
    GLfloat light0_position[] = {0.15,1.0,0.8,0.0};
    
    glMaterialfv (GL_FRONT, GL_SPECULAR, mat_specular);
    glMaterialfv (GL_FRONT, GL_SHININESS, mat_shininess);
    glLightfv (GL_LIGHT0, GL_DIFFUSE, light0_diffuse);
    glLightfv (GL_LIGHT0, GL_POSITION, light0_position);
    
    glEnable (GL_LIGHTING);
    glEnable (GL_LIGHT0);
   // 
   //glCullFace(GL_FRONT);
   // glEnable(GL_CULL_FACE);
}

int main(int argc, char** argv)
{
    glutInit (&argc, argv);
    glutInitDisplayMode (GLUT_SINGLE | GLUT_RGB);
    glutInitWindowSize (500, 500);
    glutInitWindowPosition (100, 100);
    glutCreateWindow ("Icosahedron");
    init ();
    glutDisplayFunc (display);
    glutMainLoop();
    return 0;
}
