/* An Icosahedron can be considered as an approximation to a sphere
with increasing number of triangular faces   */

#include <GL/glut.h>
#include <math.h>
#include <stdio.h>

#define x .525731112119133606
#define y .0
#define z .850650808352039932

void normalize(float []);
void drawtriangle(float *, float *, float *);
void subdivide(float *, float *, float *);

void display (void)
{
    static GLfloat vdata[12][3] = {
        {-x,y,z}, {x,y,z}, {-x,y,-z}, {x,y,-z},
        {y,z,x}, {y,z,-x}, {y,-z,x}, {y,-z,-x},
        {z,x,y}, {-z,x,y}, {z,-x,y}, {-z,-x,y} };
 
    static GLuint tindices[20][3] = {
        {0,4,1}, {0,9,4}, {9,5,4}, {4,5,8}, {4,8,1}, 
        {8,10,1}, {8,3,10}, {5,3,8}, {5,2,3}, {2,7,3}, 
        {7,10,3}, {7,6,10}, {7,11,6}, {11,0,6}, {0,1,6}, 
        {6,1,10}, {9,0,11}, {9,11,2}, {9,2,5}, {7,2,11} };   
    int i;
    glClearColor (1.0, 1.0, 1.0, 0.0);
    glClear(GL_COLOR_BUFFER_BIT);
    glColor3f(1.0, 1.0, 1.0);
    
    glLineWidth(5.0);
    glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
    for (i = 0; i < 20; i++) {
        subdivide(&vdata[tindices[i][0]][0],
            &vdata[tindices[i][1]][0],
            &vdata[tindices[i][2]][0]);
    }
    glFlush();
}

void normalize(float v[3]){
    GLfloat d = sqrt(v[0]*v[0]+v[1]*v[1]+v[2]*v[2]);
    v[0] /=d; v[1] /=d; v[2] /=d;
}
       
void subdivide(float *v1, float *v2, float *v3)
{
    GLfloat v12[3], v23[3], v31[3];
    GLint j;
    for (j=0; j < 3; j++) {
        v12[j] = (v1[j]+v2[j]);
        v23[j] = (v2[j]+v3[j]);
        v31[j] = (v3[j]+v1[j]);
    }
    normalize(v12);
    normalize(v23);
    normalize(v31);
    drawtriangle(v1,v12,v31);
    drawtriangle(v2,v23,v12);
    drawtriangle(v3,v31,v23);
    drawtriangle(v12,v23,v31);
}

void drawtriangle(float *v1, float *v2, float *v3)
{
    glBegin(GL_TRIANGLES);
        glNormal3fv(v1); glVertex3fv(v1);
        glNormal3fv(v2); glVertex3fv(v2);
        glNormal3fv(v3); glVertex3fv(v3);
    glEnd();
}

void init(void)
{
    GLfloat mat_specular[] = {1.0,1.0,1.0,0.0};
    GLfloat mat_shininess[] = {20.0};
    GLfloat light0_diffuse[] = {1.0,1.0,1.0,1.0};
    GLfloat light0_position[] = {1.0,1.0,1.0,0.0};
    
    glMaterialfv (GL_FRONT, GL_SPECULAR, mat_specular);
    glMaterialfv (GL_FRONT, GL_SHININESS, mat_shininess);
    glLightfv (GL_LIGHT0, GL_DIFFUSE, light0_diffuse);
    glLightfv (GL_LIGHT0, GL_POSITION, light0_position);
    
    glEnable (GL_LIGHTING);
    glEnable (GL_LIGHT0);
}

int main(int argc, char** argv)
{
    glutInit (&argc, argv);
    glutInitDisplayMode (GLUT_SINGLE | GLUT_RGB);
    glutInitWindowSize (500, 500);
    glutInitWindowPosition (100, 100);
    glutCreateWindow ("Polygonal Approximation to Sphere");
    init ();
    glutDisplayFunc (display);
    glutMainLoop();
    return 0;
}

